@@ -37,11 +37,19 @@ static const char* s_flickr_rest = "http://www.flickr.com/services/rest/";
 static const char* s_apikey = "64dfce7e96302f412c7f0ece0897b1d2";
 static const char* s_secret = "d7458fd6f08296e1";
 
+static QHash< QString, PhotoListType >  s_photoListTypes;
+
 FlickrEngine::FlickrEngine( QObject *p_parent, const QVariantList &p_args )
 : DataEngine( p_parent, p_args )
+, m_waiting_for_retrieval( false )
 , m_timer( 0 )
 , m_type( INTERESTINGNESS )
 {
+  s_photoListTypes[ "interestingness" ] = INTERESTINGNESS;
+  s_photoListTypes[ "favorites" ] = FAVORITES;
+  s_photoListTypes[ "photoset" ] = PHOTOSET;
+  s_photoListTypes[ "tag" ] = TAG;
+  s_photoListTypes[ "location" ] = LOCATION;
 }
 
 FlickrEngine::~FlickrEngine()
@@ -214,6 +222,7 @@ void FlickrEngine::retrievePhotos()
     case FAVORITES: retrieveUserFavoriteList(); break;
     case PHOTOSET: retrievePhotoset( m_config["photoset-id"].toString() ); break;
     case TAG: retrievePhotosByCluster( m_config["tag"].toString(), m_config["cluster-id"].toString() ); break;
+    case LOCATION: retrievePhotosByGeo( m_config["location-lon" ].toDouble(), m_config["location-lat"].toDouble(), m_config["location-accuracy"].toInt() ); break;
     default: retrieveInterestingnessList(); break;
   }
 }
@@ -237,6 +246,8 @@ void FlickrEngine::retrieveUserFavoriteList()
 
 void FlickrEngine::listJobDone( KJob *p_job )
 {
+  m_waiting_for_retrieval = false;
+
   QDomDocument doc;
   if ( processResponse( p_job, &doc ) )
   {
@@ -249,9 +260,9 @@ void FlickrEngine::listJobDone( KJob *p_job )
       {
         retrieveAuthor( *m_photo_it );
       }
-      nextPhoto();
     }
   }
+  nextPhoto();
 }
 
 void FlickrEngine::cleanCache()
@@ -326,7 +337,7 @@ void FlickrEngine::processPhoto( const QDomNode &p_photo )
   }
 
   photo.photoRetrieved = false;
-   // no need to wait for the author retrieval when we're doing photosets. we already know the author and we won't try to retrieve the author.
+  // no need to wait for the author retrieval when we're doing photosets. we already know the author and we won't try to retrieve the author.
   photo.authorRetrieved = m_type == PHOTOSET;
 
   QString path = "flickrop/";
@@ -341,10 +352,10 @@ void FlickrEngine::processPhoto( const QDomNode &p_photo )
 KUrl FlickrEngine::constructPhotoURL( const QString &p_id, const QString &p_farm, const QString &p_server, const QString &p_secret )
 {
   QString base = QString( "http://farm%1.static.flickr.com/%2/%3_%4" )
-               .arg( p_farm )
-               .arg( p_server )
-               .arg( p_id )
-               .arg( p_secret );
+    .arg( p_farm )
+    .arg( p_server )
+    .arg( p_id )
+    .arg( p_secret );
   if ( m_size == "o" )
   {
     // TODO: original secret?
@@ -384,11 +395,15 @@ void FlickrEngine::authorJobDone( KJob *p_job )
     {
       if ( (*it).id == photoid )
       {
-	break;
+        break;
       }
     }
 
-    Q_ASSERT( it != m_photos.end() );
+    if ( it == m_photos.end() )
+    {
+      // the photo disappeared, perhaps a new list was retrieved in the meantime
+      return;
+    }
 
     (*it).author.username = doc.elementsByTagName( "username" ).item(0).toElement().text();
     (*it).author.profileUrl = doc.elementsByTagName( "profileurl" ).item(0).toElement().text();
@@ -408,9 +423,22 @@ void FlickrEngine::nextPhoto()
   // in case we force a photo update (i.e. the timer hasn't timed out yet)
   m_timer->stop();
 
+  // the boolean makes sure to call this only once. for the frantics out there hitting Next Photo repeatedly.
+  if ( !m_waiting_for_retrieval && m_photos.isEmpty() )
+  {
+    // retrieving the photos has failed somehow. let's try again in 5
+    // minutes.
+    QTimer::singleShot( 5 * 60 * 1000, this, SLOT( retrievePhotos() ) );
+    m_waiting_for_retrieval = true; return;
+  }
+  else if ( m_waiting_for_retrieval )
+  {
+    // waiting, don't do anything now
+    return;
+  }
+
   if ( m_photo_it == m_photos.end() )
   {
-    kDebug() << "retrieving new photos" << endl;
     // we ran out of photos, retrieve new photos
     retrievePhotos();
     return;
@@ -423,7 +451,6 @@ void FlickrEngine::nextPhoto()
   }
   else if ( (*m_photo_it).authorRetrieved )
   {
-    kDebug() << "show from cache" << endl;
     // show photo from cache
     setPhoto( *m_photo_it );
   }
@@ -467,7 +494,7 @@ void FlickrEngine::photoJobDone( KJob *p_job )
     {
       if ( (*it).id == photoid )
       {
-	break;
+        break;
       }
     }
 
@@ -528,23 +555,7 @@ void FlickrEngine::setConfig( const QVariant &p_config )
   m_token = m_config.value( "token" ).toString();
   setData( "authentication", "token", m_token );
 
-  QString type = m_config.value( "listtype" ).toString();
-  if ( type == "interestingness" )
-  {
-    m_type = INTERESTINGNESS;
-  }
-  else if ( type == "favorites" )
-  {
-    m_type = FAVORITES;
-  }
-  else if ( type == "photoset" )
-  {
-    m_type = PHOTOSET;
-  }
-  else if ( type == "tag" )
-  {
-    m_type = TAG;
-  }
+  m_type = s_photoListTypes[ m_config.value( "listtype" ).toString() ];
 
   if ( m_config.value( "check_token" ).toBool() )
   {
@@ -604,7 +615,7 @@ void FlickrEngine::clusterJobDone( KJob *p_job )
       QStringList tags;
       for ( uint j = 0; j < tagElements.length(); ++j )
       {
-	tags += tagElements.item( j ).toElement().text();
+        tags += tagElements.item( j ).toElement().text();
       }
 
       QString key = QString( "cluster%1" ).arg( i );
@@ -655,8 +666,9 @@ void FlickrEngine::retrievePhotosets( const QString &p_nsid )
 {
   Params p;
   p["method"] = "flickr.photosets.getList";
+  p["auth_token"] = m_token;
   p["user_id"] = p_nsid.isEmpty() ? m_nsid : p_nsid;
-  KIO::StoredTransferJob *job = KIO::storedGet( constructApiUrl( p, false ), KIO::NoReload, KIO::HideProgressInfo );
+  KIO::StoredTransferJob *job = KIO::storedGet( constructApiUrl( p, true ), KIO::NoReload, KIO::HideProgressInfo );
   connect( job, SIGNAL( result( KJob * ) ), SLOT( photosetJobDone( KJob * ) ) );
 }
 
@@ -670,7 +682,7 @@ void FlickrEngine::photosetJobDone( KJob *p_job )
 
     if ( photosets.isEmpty() )
     {
-      setError( i18n( "No photosets found for this user." ), ERR_MESSAGEBOX );
+      setError( i18n( "No (public) photosets were found for this user. This could occur when the user has uploaded restricted content. Please follow these steps if you want to access restricted content:<ol><li>Create an account at Flickr if you don't have one already;</li><li>Disable SafeSearch. Go to <em>You</em>\342\200\243<em>Your&nbsp;Account</em>\342\200\243<em>Privacy&nbsp;&amp;&nbsp;Permissions</em>\342\200\243<em>Edit&nbsp;Search&nbsp;settings</em>\342\200\243<em>SafeSearch&nbsp;off</em></li><li>Authenticate Flickr On Plasma in the Authentication section.</li></ul>" ), ERR_MESSAGEBOX );
       return;
     }
 
@@ -691,10 +703,23 @@ void FlickrEngine::retrievePhotoset( const QString &p_photoset_id )
 {
   Params p;
   p["method"] = "flickr.photosets.getPhotos";
+  p["auth_token"] = m_token;
   p["media"] = "photos";
   p["photoset_id"] = p_photoset_id;
-  KIO::StoredTransferJob *job = KIO::storedGet( constructApiUrl( p, false ), KIO::NoReload, KIO::HideProgressInfo );
+  KIO::StoredTransferJob *job = KIO::storedGet( constructApiUrl( p, true ), KIO::NoReload, KIO::HideProgressInfo );
+  connect( job, SIGNAL( result( KJob * ) ), SLOT( listJobDone( KJob * ) ) );
+}
+
+void FlickrEngine::retrievePhotosByGeo( qreal p_longitude, qreal p_latitude, int p_accuracy )
+{
+  Params p;
+  p["method"] = "flickr.photos.search";
+  p["lon"] = QString::number( p_longitude );
+  p["lat"] = QString::number( p_latitude );
+  p["accuracy"] = QString::number( p_accuracy );
+  KIO::StoredTransferJob *job = KIO::storedGet( constructApiUrl( p, true ), KIO::NoReload, KIO::HideProgressInfo );
   connect( job, SIGNAL( result( KJob * ) ), SLOT( listJobDone( KJob * ) ) );
 }
 
 K_EXPORT_PLASMA_DATAENGINE(flickrop, FlickrEngine)
+
